import 'package:flutter/material.dart';
import 'package:flutter_flow_chart/flutter_flow_chart.dart';
import 'package:oc_front/core/services/perms_service.dart';
import 'package:oc_front/main.dart';
import 'package:oc_front/models/search.dart';
import 'package:oc_front/models/workflow.dart';
import 'package:oc_front/pages/workflow.dart';
import 'package:oc_front/widgets/forms/sub_keys_forms.dart';
import 'package:oc_front/widgets/forms/sub_expose_forms.dart';
import 'package:oc_front/widgets/forms/sub_map_forms.dart';
import 'package:oc_front/widgets/inputs/sub_text_input.dart';

Map<String, Map<String, AbstractItem>> proxyWfItem = {};
// ADD EXPOSE
class ProcessingFormsWidget extends StatefulWidget {
  String elementID;
  ProcessingItem item;
  Dashboard dash;
  ProcessingFormsWidget ({ super.key, required this.item, required this.dash, required this.elementID });
  @override ProcessingFormsWidgetState createState() => ProcessingFormsWidgetState();
}
class ProcessingFormsWidgetState extends State<ProcessingFormsWidget> {
  Widget getInputAndOutputVariableForms(bool readOnly) {
    var inList = widget.dash.GetArrowByElementID(widget.elementID, true);
    var outList = widget.dash.GetArrowByElementID(widget.elementID, false);
    List<Widget> res = [];
    List<GraphItem> inItems = [];
    List<GraphItem> outItems = [];
    for (var inItem in inList) {
      var element = widget.dash.getElement(inItem.fromID.substring(0,36));
      if (element == null) { continue; }
      var g = GraphItem();
      g.fromDashboard(element.serialize());
      inItems.add(g);
    }
    for (var outItem in outList) {
      var element = widget.dash.getElement(outItem.toID.substring(0,36));
      if (element == null) { continue; }
      var g = GraphItem();
      g.fromDashboard(element.serialize());
      outItems.add(g);
    }
    if (inItems.isNotEmpty) {
      res.add(SubKeysMapFormsWidget(dash: widget.dash, isInput: true, item: widget.item, elementID: widget.elementID, 
        categoryKey: "container", varKey: "env", graphItems: inItems, readOnly: readOnly));
    }
    if (outItems.isNotEmpty) {
      res.add(SubKeysMapFormsWidget(dash: widget.dash, isInput: false, item: widget.item, elementID: widget.elementID, 
        categoryKey: "container", varKey: "env", graphItems: outItems, readOnly: readOnly));
    }
    return Column( children: res );
  }

  @override Widget build(BuildContext context) {
    bool readOnly = !PermsService.getPerm(Perms.WORKFLOW_EDIT);
    List<Widget> categories = [];
    var l = widget.item.model?.model?.keys ?? [];
    for (var child in l) {
      var sub = widget.item.model!.model![child]!;
      List<Widget> children = [];
      for (var st in sub.keys) {  
        if (sub[st]!.type?.contains("map") ?? false) {
          children.add(
            SubMapFormsWidget(dash: dash, empty: children.isEmpty, item: widget.item, 
                          readOnly: readOnly,
                          elementID: widget.elementID, categoryKey: child, varKey: st)
          );
        } else if (sub[st]!.type == "string") {
          children.add(SubTextInputWidget(subkey: st, 
            readOnly: readOnly,
            initialValue: widget.item.getVariable([child, st], widget.item.serialize()), 
            width: 180, empty: children.isEmpty, change: (value) { 
              widget.item.model ?? Model();
              Future.delayed(const Duration(seconds: 2), () {
                if (widget.item.getVariable([child, st], widget.item.serialize()) == value) {
                  dash.saveDash(dash.id);
                }
              });
              var el = dash.getElement(widget.elementID);
              widget.item = widget.item.deserialize(widget.item.setVariable([child, st], value, widget.item.serialize())) as dynamic;
              el!.element = widget.item as dynamic;
            })
          );
        }  
      }
      categories.add(Container(
          padding: const EdgeInsets.symmetric(vertical: 10),
          width: 180,
          child: Column( 
            mainAxisAlignment: MainAxisAlignment.center,
            children: [
              Text("<${child.toUpperCase()}>", style: const TextStyle(fontSize: 13, fontWeight: FontWeight.bold), textAlign: TextAlign.center),
              Padding(padding: const EdgeInsets.only(bottom: 10), child: SubTextInputWidget(subkey: "image", width: 180, empty: false, change: (value) {}, 
                initialValue: widget.item.container?.image, readOnly: true,)),
              ...children,
              getInputAndOutputVariableForms(readOnly),
            ],)
        ));
    }
    // EXPOSE 
    categories.add(Container(
          padding: const EdgeInsets.symmetric(vertical: 10),
          width: 180,
          child: Column( 
            mainAxisAlignment: MainAxisAlignment.center,
            children: [
              const Padding(padding: EdgeInsets.only(bottom: 5), child: Text("<EXPOSE>", 
                style: TextStyle(fontSize: 13, fontWeight: FontWeight.bold), textAlign: TextAlign.center)),
              readOnly ? Container() : Row( children: [
                InkWell( onTap: () {
                    widget.item.expose.add(Expose());
                    var el = dash.getElement(widget.elementID);
                    el!.element = widget.item as dynamic;
                    setState(() {});
                  }, child:
                  Container( margin: const EdgeInsets.only(top: 5),
                    decoration: BoxDecoration(borderRadius: BorderRadius.circular(5), border: Border.all(color: Colors.grey, width: 1)),
                    width: 125, height: 30,
                    child: const Row( mainAxisAlignment: MainAxisAlignment.center,
                      children: [ Padding( padding: EdgeInsets.only(right: 5), child: Icon(Icons.add)), Text("add expose")]),
                  )
                ),
                InkWell( onTap: () {
                  if (widget.item.expose.isEmpty) { return; }
                    widget.item.expose = widget.item.expose.sublist(0, widget.item.expose.length - 1);
                    var el = dash.getElement(widget.elementID);
                    el!.element = widget.item as dynamic;
                    setState(() {});
                  }, child:
                  Container( margin: const EdgeInsets.only(left: 5, top: 5),
                    decoration: BoxDecoration(borderRadius: BorderRadius.circular(5), border: Border.all(color: Colors.grey, width: 1)),
                    width: 45, height: 30,
                    child: const Row( mainAxisAlignment: MainAxisAlignment.center,
                      children: [ Icon(Icons.delete, color: Colors.black) ]),
                  )
                )
              ]),
            ],)
        ));
    for (var expose in widget.item.expose) {
      categories.add(SubExposeFormsWidget( readOnly: readOnly, width: 180, dash: dash, empty: categories.isEmpty, 
      item: expose, elementID: widget.elementID));
    }
    return SizedBox( height: getHeight(context) - 330, child: SingleChildScrollView( child: Column( 
      children: categories )) );
  }
}