import 'package:alert_banner/exports.dart';
import 'package:cron/cron.dart';
import 'package:intl/intl.dart' as intl;
import 'package:flutter/material.dart';
import 'package:flutter_flow_chart/flutter_flow_chart.dart';
import 'package:datetime_picker_formfield/datetime_picker_formfield.dart';
import 'package:oc_front/core/models/shared_workspace_local.dart';
import 'package:oc_front/core/services/perms_service.dart';
import 'package:oc_front/core/services/specialized_services/workflow_service.dart';
import 'package:oc_front/core/services/specialized_services/check_service.dart';
import 'package:oc_front/main.dart';
import 'package:oc_front/pages/shared.dart';
import 'package:oc_front/pages/workflow.dart';
import 'package:oc_front/widgets/dialog/alert.dart';
import 'package:oc_front/widgets/dialog/confirm_box.dart';
import 'package:oc_front/widgets/inputs/shallow_text_input.dart';

class SchedulerFormsWidget extends StatefulWidget {
  Dashboard item;
  String purpose = ""; 
  bool? booking;
  String? error;
  String? errorEndDate;
  String? errorCron;
  Function validate = () {};
  SchedulerFormsWidget ({ super.key, required this.item, });
  @override SchedulerFormsWidgetState createState() => SchedulerFormsWidgetState();
}
class SchedulerFormsWidgetState extends State<SchedulerFormsWidget> {
  CheckService check = CheckService();
  void save(List<GlobalKey<FormFieldState>> formKeys) {
    dash.scheduleActive = !dash.scheduleActive;
    widget.error = null;
    widget.errorEndDate = null;
    widget.errorCron = null;
    for (var k in formKeys) {
      if (k.currentState != null) {
        if (!k.currentState!.validate() && dash.scheduleActive) {
          dash.scheduleActive = !dash.scheduleActive;
          return;
        } else { k.currentState!.save();}
      }
    }
    DateTime now = DateTime.now().add(const Duration(minutes: 1));
    if (dash.scheduler["start"] == null || DateTime.parse(dash.scheduler["start"]!).isBefore(now)) {
      dash.scheduler["start"] = now.toUtc().toIso8601String();
      if (dash.scheduler["end"] != null) {
        dash.scheduler["end"] = now.add(const Duration(minutes: 1)).toUtc().toIso8601String();
      }
    }
    widget.item.saveDash(widget.item.id);
  }
  void checkBooking(List<GlobalKey<FormFieldState>> formKeys, void Function(List<GlobalKey<FormFieldState>> )? f){
    if (widget.item.scheduler["start"] == null) {
      DateTime now = DateTime.now().add(const Duration(minutes: 5));
      widget.item.scheduler["start"] = now.toUtc().toIso8601String();
    }
    var s = DateTime.parse(widget.item.scheduler["start"]).toUtc().toIso8601String();
    var e = "";
    if (widget.item.scheduler["end"] == null) {
      e = DateTime.parse(widget.item.scheduler["start"]).add(const Duration(seconds: 5)).toUtc().toIso8601String();
    } else {
      e = DateTime.parse(widget.item.scheduler["end"]).toUtc().toIso8601String();
    }
    check.search(context, [widget.item.id ?? "", s.substring(0, 19), e.substring(0, 19)], {}).then(
      (v) {
        if (v.data == null) { return; }
        widget.booking = v.data!.is_available;
        if (v.data!.is_available) {
          if (f != null) { f(formKeys); 
          } else {
            showAlertBanner( context, () {}, 
              const InfoAlertBannerChild(text: "no booking found at this date"),// <-- Put any widget here you want!
              alertBannerLocation:  AlertBannerLocation.bottom,);
          } 
        } else {
          showAlertBanner( context, () {}, 
          const AlertAlertBannerChild(text: "booking found at this date"),// <-- Put any widget here you want!
           alertBannerLocation:  AlertBannerLocation.bottom,);
        }
        setState(() {});
      }
    );
  }

  @override Widget build(BuildContext context) {
    bool isService = true;
    try {
      widget.item.elements.firstWhere((element) => (element.element?.serialize()["is_service"] ?? false) == true);
    } catch (e) {
      isService = false;
    }
    bool readOnly = !PermsService.getPerm(Perms.WORKFLOW_EDIT);
    DateTime? start;
    DateTime? end;

    Duration delayed = const Duration(minutes: 5);
    if (widget.item.scheduler["start"] != null) {
      start = DateTime.parse(widget.item.scheduler["start"]!);
      if (start.isBefore(DateTime.now()) && !dash.scheduleActive) {
        start = DateTime.now().add(const Duration(minutes: 5));
        widget.item.scheduler["start"] = start.toUtc().toIso8601String();
      }
      if (start.isBefore(DateTime.now())) {
        // get difference between now and start
        delayed = start.difference(DateTime.now());
      }
    }
    if (widget.item.scheduler["end"] != null) {
      end = DateTime.parse(widget.item.scheduler["end"]!);
      if (end.isBefore(DateTime.now()) && !dash.scheduleActive) {
        end = DateTime.now().add(const Duration(minutes: 5));
        widget.item.scheduler["end"] = end.toUtc().toIso8601String();
      }
      if (end.isBefore(DateTime.now())) {
        // get difference between now and start
        delayed = end.difference(DateTime.now());
        Future.delayed(delayed, () {
          WorkflowFactory.key.currentState?.setState(() { });
        });
      }
    }
    List<GlobalKey<FormFieldState>> formKeys = [GlobalKey<FormFieldState>(), GlobalKey<FormFieldState>(), GlobalKey<FormFieldState>(), GlobalKey<FormFieldState>()];
    var shallow = ShallowTextInputWidget(
        width: 200 - 1, readOnly: readOnly,
        current: widget.item.name,
        type: CollaborativeAreaType.workflow,
        canRemove: PermsService.getPerm(Perms.WORKFLOW_DELETE) ? (p0) => p0 != null && p0.isNotEmpty : null,
        remove: PermsService.getPerm(Perms.WORKFLOW_DELETE) ? (p0) async {
          showDialog(context: context, builder: (context) {
            return ConfirmBoxWidget(
              purpose: "Are you sure you want to delete this workflow?",
              validate: () async {
                await WorflowService().delete(context, widget.item.id ?? "", {}).then((value) {
                  dash.id = null;
                  dash.name = "";
                  dash.isOpened = false;
                  dash.clear();
                  dash.chartKey.currentState?.widget.flowChart.setState(() { });
                });
              },
            );
          });
        } : null,
      );
    shallow.change =(p0) => Future.delayed( const Duration(seconds: 2), () async {
      if (shallow.compare == p0) {
        await WorflowService().put(context, widget.item.id ?? "", { "name" : p0 }, {});
      } else {
        shallow.compare = p0;
      }
    });
    return Column( children: [
      Container( padding: const EdgeInsets.all(10), width: 200, height: 60,
        decoration: const BoxDecoration(border: Border(bottom: BorderSide(color: Colors.grey, width: 1))),
        child: const Column( mainAxisAlignment: MainAxisAlignment.center, children: [
          Text("WORKFLOW INFO", style: TextStyle(fontSize: 15, fontWeight: FontWeight.bold), textAlign: TextAlign.center),
          Text("<general>", style: TextStyle(fontSize: 12), textAlign: TextAlign.center),
      ])),
      widget.item.name == "" ? Container() : Container( 
        decoration: BoxDecoration( border: Border(
          left: BorderSide(color: midColor, width: 1),
          bottom: const BorderSide(color: Colors.grey))),
        child: shallow ),
      const SizedBox(height: 20, width: 200 ),
      isService ? Text("Warning a processing is a service, if no end execution it will run forever.") : Container(),
      Tooltip( message: "executions name",
        child: Container( height: 40, margin: const EdgeInsets.only(top: 5),
                              padding: const EdgeInsets.symmetric(vertical: 5, horizontal: 10),
                              child: TextFormField( key: formKeys[0], readOnly: readOnly,
                              initialValue: "${widget.item.scheduler["name"] ?? "${widget.item.name}_executions"}",
                              enabled: !dash.scheduleActive && !readOnly,
                              onChanged: (value) { 
                                Future.delayed(const Duration(seconds: 100), () {
                                  if (widget.item.scheduler["name"] == value) { 
                                    widget.item.saveDash(widget.item.id);  
                                  }
                                });
                                widget.item.scheduler["name"] = value;
                              },
                              onSaved: (value) { 
                                widget.item.scheduler["name"] = value ?? "${widget.item.scheduler["name"] ?? "${widget.item.name}_executions"}";
                              },
                              validator: (value) {
                                if (value == null || value.isEmpty) {
                                  setState(() { widget.error = 'missing name';  });   
                                }
                                return value == null || value.isEmpty ? "not empty" :  null;
                              },
                              style: const TextStyle(fontSize: 12),
                              decoration: InputDecoration(
                                floatingLabelBehavior: FloatingLabelBehavior.always,
                                fillColor: Colors.white,
                                filled: true,
                                hintText: "enter executions name...",
                                labelText: "executions name*",
                                hintStyle: TextStyle(fontSize: 10),
                                labelStyle: TextStyle(fontSize: 10),
                                focusedErrorBorder: OutlineInputBorder(borderSide: BorderSide(color: widget.item.error != null || widget.error != null ? Colors.red : Colors.black)),
                                errorBorder: OutlineInputBorder(borderSide: BorderSide(color: widget.item.error != null || widget.error != null ? Colors.red : Colors.black)),
                                focusedBorder: OutlineInputBorder(borderSide: BorderSide(color: widget.item.error != null || widget.error != null ? Colors.red : Colors.black)),
                                enabledBorder: OutlineInputBorder(borderSide: BorderSide(color: widget.item.error != null || widget.error != null ? Colors.red : Colors.grey)),
                                border: OutlineInputBorder(borderSide: BorderSide(color: widget.item.error != null || widget.error != null ? Colors.red : Colors.grey)),
                                contentPadding: EdgeInsets.symmetric(horizontal: 10, vertical: 5),
                              ),
                          ))),
      Tooltip( message: "start executions",
        child: Container( height: 40, margin: const EdgeInsets.only(top: 5),
                              padding: const EdgeInsets.symmetric(vertical: 5, horizontal: 10),
                              child: DateTimeField( key: formKeys[1],
                                enabled: !dash.scheduleActive && !readOnly,
                                resetIcon: null,
                                onShowPicker: (context, currentValue) async { 
                                  var date = await showDatePicker(                                    
                                    builder: (BuildContext context, Widget? child) {
                                      Widget w = Theme(
                                        data: ThemeData(
                                          cardTheme: CardTheme(elevation: 0, shape: RoundedRectangleBorder(borderRadius: BorderRadius.circular(0))),
                                          dialogTheme: DialogTheme(elevation: 0, shape: RoundedRectangleBorder(borderRadius: BorderRadius.circular(0))),
                                          colorScheme: ColorScheme.light(
                                            background: midColor,
                                            tertiary: Colors.grey,
                                            secondary: Colors.grey,
                                            primary: Colors.black),
                                        ),
                                        child: child ?? Container(),
                                      );
                                      return w;
                                    },
                                    context: context,
                                    firstDate: dash.scheduleActive ? DateTime(1900) : DateTime.now().add(const Duration(minutes: 5)),
                                    initialDate: DateTime.parse( start?.toLocal().toIso8601String() 
                                      ?? currentValue?.toIso8601String() 
                                      ?? DateTime.now().add(const Duration(minutes: 5)).toUtc().toIso8601String()).toLocal(),
                                    lastDate: DateTime(2100)
                                  );
                                  if (date != null) {
                                    TimeOfDay? time;    
                                    var count = 0;             
                                    while(date!.microsecondsSinceEpoch <= (DateTime.now().microsecondsSinceEpoch) || time == null) {
                                      if (count > 0) {
                                        showAlertBanner( context, () {}, 
                                        const AlertAlertBannerChild(
                                          text: "must be at least 1 minute from now to let system check info"),// <-- Put any widget here you want!
                                         alertBannerLocation:  AlertBannerLocation.bottom,);
                                      }
                                      time = await showTimePicker(context: context, 
                                        initialTime: TimeOfDay(hour: date.hour, minute: date.minute),
                                        builder: (BuildContext context, Widget? child) {
                                        Widget w = Theme(
                                          data: ThemeData(
                                            cardTheme: CardTheme(elevation: 0, shape: RoundedRectangleBorder(borderRadius: BorderRadius.circular(0))),
                                            dialogTheme: DialogTheme(elevation: 0, shape: RoundedRectangleBorder(borderRadius: BorderRadius.circular(0))),
                                            colorScheme: ColorScheme.light(
                                              background: midColor,
                                              tertiary: Colors.grey,
                                              secondary: Colors.grey,
                                              primary: Colors.black),
                                          ),
                                          child: child ?? Container(),
                                        );
                                        return w;
                                      },
                                      );
                                      if (time == null) { return DateTime.now().add( const Duration(minutes: 1)); }
                                      count++;
                                      date = DateTime(date.year, date.month, date.day, time.hour, time.minute);
                                      widget.item.scheduler["start"] = date.toUtc().toIso8601String();  
                                    }
                                    widget.item.saveDash(widget.item.id);   
                                  }
                                  return date;
                                },
                                format: intl.DateFormat('y-M-dd HH:mm:ss'),  
                                initialValue: start?.toLocal() ?? DateTime.now(),
                                onChanged: (value) {  },
                                validator: (value) {
                                  return value == null ? "not empty" :  null;
                                },
                                style: const TextStyle(fontSize: 12),
                                decoration: const InputDecoration(
                                  floatingLabelBehavior: FloatingLabelBehavior.always,
                                  fillColor: Colors.white,
                                  filled: true,
                                  alignLabelWithHint: false,
                                  hintText: "enter start executions...",
                                  labelText: "start executions*",
                                  hintStyle: TextStyle(fontSize: 10),
                                  labelStyle: TextStyle(fontSize: 10),
                                  enabledBorder: OutlineInputBorder(borderSide: BorderSide(color: Colors.grey)),
                                  border: OutlineInputBorder(borderSide: BorderSide(color: Colors.grey)),
                                  contentPadding: EdgeInsets.symmetric(horizontal: 10, vertical: 5),
                                ),
                          ))),
      Tooltip( message: "end executions",
        child: Container( height: 40, margin: const EdgeInsets.only(top: 5), 
                              padding: const EdgeInsets.symmetric(vertical: 5, horizontal: 10),
                              child: DateTimeField( key: formKeys[2],
                                enabled: !dash.scheduleActive && !readOnly,
                                validator: (value) {
                                  if (value == null && widget.item.scheduler["cron"] != null && widget.item.scheduler["cron"].isNotEmpty) {
                                    setState(() {
                                      widget.errorEndDate = 'missing start date';
                                    });   
                                  }
                                  return value == null && widget.item.scheduler["cron"] != null && widget.item.scheduler["cron"].isNotEmpty ? "not empty" :  null;
                                },
                                onChanged: (value) { 
                                  if (value == null) {
                                    widget.item.scheduler.remove("end");
                                  }
                                },
                                resetIcon: const Icon(Icons.close, size: 15),
                                onShowPicker: (context, currentValue) async { 
                                  var date = await showDatePicker(                                    
                                    builder: (BuildContext context, Widget? child) {
                                      Widget w = Theme(
                                        data: ThemeData(
                                          cardTheme: CardTheme(elevation: 0, shape: RoundedRectangleBorder(borderRadius: BorderRadius.circular(0))),
                                          dialogTheme: DialogTheme(elevation: 0, shape: RoundedRectangleBorder(borderRadius: BorderRadius.circular(0))),
                                          colorScheme: ColorScheme.light(
                                            background: midColor,
                                            tertiary: Colors.grey,
                                            secondary: Colors.grey,
                                            primary: Colors.black),
                                        ),
                                        child: child ?? Container(),
                                      );
                                      return w;
                                    },
                                    context: context,
                                    firstDate: dash.scheduleActive ? DateTime(1900) : DateTime.now().add(const Duration(minutes: 5)),
                                    initialDate: DateTime.parse( end?.toLocal().toIso8601String() 
                                      ?? currentValue?.toIso8601String() 
                                      ?? DateTime.now().add(const Duration(minutes: 5)).toUtc().toIso8601String()).toLocal(),
                                    lastDate: DateTime(2100)
                                  );
                                  if (date != null) {
                                    // ignore: use_build_context_synchronously
                                    date = DateTime(date.year, date.month, date.day, date.hour, date.minute);
                                    TimeOfDay? time;
                                    var count = 0;
                                    while(date!.microsecondsSinceEpoch <= (DateTime.now().microsecondsSinceEpoch) 
                                      || time == null
                                      || (date.microsecondsSinceEpoch) <= (DateTime.parse(widget.item.scheduler["start"] ?? DateTime.now().toIso8601String()).microsecondsSinceEpoch)) {
                                      if (count > 0) { 
                                        showAlertBanner( context, () {},  // ignore: use_build_context_synchronously
                                        const AlertAlertBannerChild(text: "must be at least 1 minute from now to let system check info && upper starting date"),// <-- Put any widget here you want!
                                         alertBannerLocation:  AlertBannerLocation.bottom,);
                                      }
                                      // ignore: use_build_context_synchronously
                                      time = await showTimePicker(context: context, 
                                        initialTime: TimeOfDay(hour: date.hour, minute: date.minute),
                                        builder: (BuildContext context, Widget? child) {
                                        Widget w = Theme(
                                          data: ThemeData(
                                            cardTheme: CardTheme(elevation: 0, shape: RoundedRectangleBorder(borderRadius: BorderRadius.circular(0))),
                                            dialogTheme: DialogTheme(elevation: 0, shape: RoundedRectangleBorder(borderRadius: BorderRadius.circular(0))),
                                            colorScheme: ColorScheme.light(
                                              background: midColor,
                                              tertiary: Colors.grey,
                                              secondary: Colors.grey,
                                              primary: Colors.black),
                                          ),
                                          child: child ?? Container(),
                                        );
                                        return w;
                                      },
                                      );
                                      if (time == null) { return null; }
                                      count++;
                                      date = DateTime(date.year, date.month, date.day, time.hour, time.minute);
                                      widget.item.scheduler["end"] = date.toUtc().toIso8601String();   
                                    }
                                    widget.item.saveDash(widget.item.id);                          
                                  }
                                  return date;
                                },
                                format: intl.DateFormat('y-M-dd HH:mm:ss'),
                                initialValue: end?.toLocal(),
                                style: const TextStyle(fontSize: 12),
                                decoration: InputDecoration(
                                  fillColor: Colors.white,
                                  floatingLabelBehavior: FloatingLabelBehavior.always,
                                  filled: true,
                                  alignLabelWithHint: false,
                                  hintText: "enter end executions...",
                                  labelText: "end executions",
                                  hintStyle: const TextStyle(fontSize: 10),
                                  labelStyle: const TextStyle(fontSize: 10),
                                  focusedErrorBorder: OutlineInputBorder(borderSide: BorderSide(color: widget.errorEndDate != null ? Colors.red : Colors.black)),
                                  errorBorder: OutlineInputBorder(borderSide: BorderSide(color: widget.errorEndDate != null ? Colors.red : Colors.black)),
                                  focusedBorder: OutlineInputBorder(borderSide: BorderSide(color: widget.errorEndDate != null ? Colors.red : Colors.black)),
                                  enabledBorder: OutlineInputBorder(borderSide: BorderSide(color: widget.errorEndDate != null ? Colors.red : Colors.grey)),
                                  border: OutlineInputBorder(borderSide: BorderSide(color: widget.errorEndDate != null ? Colors.red : Colors.grey)),
                                  contentPadding: const EdgeInsets.symmetric(horizontal: 10, vertical: 5),
                                ),
                          ))),
      Tooltip( message: "cron command",
        child: Container( height: 40, margin: const EdgeInsets.only(top: 5),
                              padding: const EdgeInsets.symmetric(vertical: 5, horizontal: 10),
                              child: TextFormField( key: formKeys[3],
                              enabled: !dash.scheduleActive && !readOnly,
                              initialValue: widget.item.scheduler["cron"],
                              onChanged: (value) { 
                                Future.delayed(const Duration(seconds: 100), () {
                                  if (widget.item.scheduler["cron"] == value) { 
                                    widget.item.saveDash(widget.item.id);  
                                  }
                                });
                                widget.item.scheduler["cron"] = value;
                              },
                              onSaved: (value) { 
                                if (value != null) {
                                  widget.item.scheduler["cron"] = value;
                                }
                              },
                              validator: (value) { 
                                var cron = Cron();
                                if (value != null && value.isNotEmpty) {
                                  try {
                                    cron.schedule(Schedule.parse(value), () {});
                                  } catch (e) {
                                    setState(() {
                                      widget.errorCron = e.toString();
                                    }); 
                                    return "invalid cron";
                                  }
                                }
                                return null;
                              },
                              style: const TextStyle(fontSize: 12),
                              decoration: InputDecoration(
                                floatingLabelBehavior: FloatingLabelBehavior.always,
                                fillColor: Colors.white,
                                filled: true,
                                hintText: "enter cron command...",
                                labelText: "cron",
                                errorStyle: TextStyle(height: 0),
                                hintStyle: TextStyle(fontSize: 10),
                                labelStyle: TextStyle(fontSize: 10),
                                focusedErrorBorder: OutlineInputBorder(borderSide: BorderSide(color: widget.errorCron != null ? Colors.red : Colors.black)),
                                errorBorder: OutlineInputBorder(borderSide: BorderSide(color: widget.errorCron != null ? Colors.red : Colors.black)),
                                focusedBorder: OutlineInputBorder(borderSide: BorderSide(color: widget.errorCron != null ? Colors.red : Colors.black)),
                                enabledBorder: OutlineInputBorder(borderSide: BorderSide(color: widget.errorCron != null ? Colors.red : Colors.grey)),
                                border: OutlineInputBorder(borderSide: BorderSide(color: widget.errorCron != null ? Colors.red : Colors.grey)),
                                contentPadding: EdgeInsets.symmetric(horizontal: 10, vertical: 5),
                              ),
                          ))),
      Container(
        width: 200,
        height: 20,
        decoration: const BoxDecoration(border: Border(bottom: BorderSide(color: Colors.grey, width: 1))),
      ),
      const SizedBox(
        width: 200,
        height: 10,
      ),
      Tooltip( message: "check booking",
                              child: InkWell( mouseCursor: SystemMouseCursors.click, 
                              onTap: () {
                                PermsService.getPerm(Perms.WORKFLOW_BOOKING) ? checkBooking(formKeys, null) : null;
                              }, child: Container( margin: const EdgeInsets.only(bottom: 5, left: 10, right: 10),
                                decoration: BoxDecoration(borderRadius: BorderRadius.circular(5), 
                                  border: Border.all(color: widget.booking == null && !dash.scheduleActive ? (
                                    PermsService.getPerm(Perms.WORKFLOW_BOOKING) && PermsService.getPerm(Perms.WORKFLOW_EDIT) ? Colors.black : Colors.grey) : (
                                      widget.booking == true || dash.scheduleActive  ? Colors.green : redColor), width: 1)),
                                width: 200, height: 30,
                                child: Icon(
                                  Icons.verified_outlined, 
                                  color: widget.booking == null && !dash.scheduleActive ? Colors.black :  (widget.booking == true || dash.scheduleActive ? Colors.green : redColor)),
                              ))
                            ),
      Tooltip( message: dash.scheduleActive ? "unbook" : "book",
                              child: InkWell( mouseCursor: SystemMouseCursors.click, 
                              onTap: () {
                                PermsService.getPerm(Perms.WORKFLOW_BOOKING) && PermsService.getPerm(Perms.WORKFLOW_EDIT) ? (dash.scheduleActive ? setState(() {  save(formKeys); }) : checkBooking(formKeys,  save)) : null;
                              }, child: Container( margin: const EdgeInsets.symmetric(vertical: 5, horizontal: 10),
                                decoration: BoxDecoration(borderRadius: BorderRadius.circular(5), 
                                  border: Border.all(color: dash.scheduleActive ? Colors.green : ( PermsService.getPerm(Perms.WORKFLOW_BOOKING) ?Colors.black : Colors.grey ))),
                                width: 200, height: 30,
                                child: Icon(
                                  dash.scheduleActive ? Icons.cancel_schedule_send : Icons.schedule_send, color: dash.scheduleActive ? Colors.green : Colors.black),
                              ))
                            ),
      widget.item.info["shared"] != null && (widget.item.info["shared"] as List<dynamic>).isNotEmpty ? Column( children: [
        Container(
          height: 30,
          width: 200,
          decoration: const BoxDecoration(border: Border(top: BorderSide(color: Colors.grey))),
        ),
        Container( alignment: Alignment.center, padding: const EdgeInsets.symmetric(horizontal: 10),
          child:Text( textAlign: TextAlign.center, overflow: TextOverflow.ellipsis,
            style: const TextStyle(fontSize: 14, color: Colors.black, fontWeight: FontWeight.bold),
            "Workflow is shared in ${(widget.item.info["shared"] as List<dynamic>).length} workspace(s)")),
        ...(widget.item.info["shared"] as List<dynamic>).where( (e) => CollaborativeAreaLocal.getCollaborativeArea(e) != null 
        ).map((e) {
          var sw = CollaborativeAreaLocal.getCollaborativeArea(e);
          return Container( alignment: Alignment.center, padding: const EdgeInsets.symmetric(horizontal: 10, vertical: 5),
            child: Row(  children: [ 
              const Padding(padding: EdgeInsets.only(right: 10), child: Icon(Icons.work, color: Colors.grey, size: 15)),
              Text(style: const TextStyle(fontSize: 12, color: Colors.grey),
              "Workspace: ${sw != null && sw.name != null ? 
              "${sw.name!.substring(0, sw.name!.length > 15 ? 12 : sw.name!.length)}${sw.name!.length > 15 ? "..." : ""}" : ""}") ]));
        },)
      ]) : Container()
    ]);
  }
}


